@@ -20,17 +20,55 @@
 #'
 #'   - For example, `"~"` denotes all regression coefficients.
 #'
-#' It is used by function such as [est_change()] and
-#' [fit_measures_change()].
+#'   - It also supports `:=`, which can be used to select
+#'     user-defined parameters.
+#'
+#' - Label
+#'
+#'   - For example, `"ab"` denotes all parameters with this
+#'     labels defined in model syntax. It can be used to
+#'     select user-defined parameters, such as `"ab := a*b"`.
+#'
+#' It is used by functions such as [est_change()].
+#'
+#' ## Multisample model
+#'
+#' If a model has more than one group, a specification
+#' specified as in a single sample model denotes the same
+#' parameters in all group.
+#'
+#'  - For example, `"f1 =~ x2"` denote the factor loading of
+#'    `x2` on `f1` in all groups. `"~~"` denotes covariances
+#'    and error covariances in all groups.
+#'
+#' There are two ways to select parameters only in selected
+#' groups. First, the syntax to fix parameter values
+#' can be used, with `NA` denoting parameters to be selected.
+#'
+#'   - For example, `"f2 =~ c(NA, 1, NA) * x5"` select the
+#'     factor loadings of `x5` on `f2` in the first and third
+#'     groups.
+#'
+#' Users can also add ".grouplabel" to a specification,
+#' `grouplabel` being the group label of a group (the one
+#' appears in [summary()], not the one of the form `".g2"`,
+#' `"g3"`, etc.).
+#'
+#'   - For example, `"f2 =~ x5.Alpha"` denotes the factor loading
+#'     of `x5` on `f2` in the group `"Alpha"`.
 #'
 #' @return
-#' A numeric vector of the ids in the column "free" in the
-#' parameter table of the fit object.
+#' A numeric vector of the ids. If `where` is `"partable"`,
+#' the ids are row numbers. If `where` is `"coef"`,
+#' the ids are the positions in the vector.
 #'
 #' @param pars A character vector of parameters specified
-#' in lavaan syntax, e.g., `"y ~ x"` and `f1 =~ x3`. Can
-#' mix specific parameters with operators. See the Details
-#' section.
+#' in lavaan syntax, e.g., `"y ~ x"` and `f1 =~ x3`. For
+#' multisample models, if only the parameters in some groups
+#' are needed, use the modifier for labeling parameters and
+#' use `NA` to denote parameters to be requested. E.g.,
+#' `f1 =~ c(NA, 0, NA, NA) * x2` denotes the loadings of
+#' `x2` on `f1` in the first, third, and fourth groups.
 #'
 #' @param fit A `lavaan`-class object. This object is used
 #' to determine the number of groups and the parameters
@@ -42,7 +80,7 @@
 #' (coefficient vector).
 #' Default is "coef".
 #'
-#' @param free_only Wether only free parameters will be
+#' @param free_only Whether only free parameters will be
 #' kept. Default is `TRUE`.
 #'
 #' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
@@ -69,6 +107,47 @@
 #' tmp <- pars_id(pars, fit = fit_ng, where = "partable")
 #' parameterTable(fit_ng)[tmp, ]
 #'
+#' # Multiple-group models
+#'
+#' dat <- sem_dat
+#' set.seed(64264)
+#' dat$gp <- sample(c("Alpha", "Beta", "Gamma"),
+#'                  nrow(dat),
+#'                  replace = TRUE)
+#'
+#' library(lavaan)
+#' sem_model <-
+#' "
+#' f1 =~  x1 + x2 + x3
+#' f2 =~  x4 + x5 + x6
+#' f3 =~  x7 + x8 + x9
+#' f2 ~   f1
+#' f3 ~   f2
+#' "
+#'
+#' fit_ng <- sem(sem_model, dat)
+#' fit_gp <- sem(sem_model, dat, group = "gp")
+#'
+#' pars <- c("f1 =~ x2", "f2 =~ x5", "f2 ~ f1")
+#' tmp <- pars_id(pars, fit = fit_ng)
+#' coef(fit_ng)[tmp]
+#' tmp <- pars_id(pars, fit = fit_ng, where = "partable")
+#' parameterTable(fit_ng)[tmp, ]
+#'
+#' pars <- c("f1 =~ x2", "f2 =~ c(NA, 1, NA) * x5")
+#' tmp <- pars_id(pars, fit = fit_gp)
+#' coef(fit_gp)[tmp]
+#' tmp <- pars_id(pars, fit = fit_gp, where = "partable")
+#' parameterTable(fit_gp)[tmp, ]
+#'
+#' pars2 <- c("f1 =~ x2", "~~.Beta", "f2 =~ x5.Gamma")
+#' tmp <- pars_id(pars2, fit = fit_gp)
+#' coef(fit_gp)[tmp]
+#' tmp <- pars_id(pars2, fit = fit_gp, where = "partable")
+#' parameterTable(fit_gp)[tmp, ]
+#' # Note that group 1 is "Beta", not "Alpha"
+#' lavInspect(fit_gp, "group.label")
+#'
 #'
 #' @export
 
@@ -90,92 +169,18 @@ pars_id <- function(pars,
                        fit = fit,
                        where = where,
                        free_only = free_only)
-    out <- sort(unique(c(ids1, ids2, ids3)))
+    ids4 <- pars_id_label(pars = pars,
+                          fit = fit,
+                          where = where,
+                          free_only = free_only)
+    out <- sort(unique(c(ids1, ids2, ids3, ids4)))
+    if (length(out) == 0) {
+        stop("No parameters selected. ",
+             "Please check the parameter argument.")
+      }
     out
   }
 
-# ###############
-# # To be added when other functions support
-# # multisample models
-# ###############
-#
-# # Details
-#
-# # Multisample model
-#
-# If a model has more than one groups, a specification
-# specified as in a single sample model denotes the same
-# parameters in all group.
-#
-# For example, `"f1 =~ x2"` denote the factor loading of
-# `x2` on `f1` in all groups. `"~~"` denotes covariances
-# and error covariances in all groups.
-#
-# There are two ways to select parameters only in selected
-# groups. First, the syntax to fix parameter values
-# can be used, with `NA` denoting parameters to be selected.
-#
-# For example, `"f2 =~ c(NA, 1, NA) * x5"` select the
-# factor loadings of `x5` on `f2` in the first and third
-# groups.
-#
-# Users can also add ".grouplabel" to a specification,
-# `grouplabel` being the group label of a group (the one
-# appears in [summary()], not the one of the form `".gp2"`,
-# `"gp3"`, etc.).
-#
-# For example, `"f2 =~ x5.Alpha"` denotes the factor loading
-# of `x5` on `f2` in the group `"Alpha"`.
-#
-# @param pars A character vector of parameters specified
-# in lavaan syntax, e.g., `"y ~ x"` and `f1 =~ x3`. For
-# multisample models, if only the parameters in some groups
-# are needed, use the modifier for labeling parameters and
-# use `NA` to denote parameters to be requested. E.g.,
-# `f1 =~ c(NA, 0, NA, NA) * x2` denotes the loadings of
-# `x2` on `f1` in the first, third, and fourth groups.
-# Example
-#
-# dat <- sem_dat
-# set.seed(64264)
-# dat$gp <- sample(c("Alpha", "Beta", "Gamma"),
-#                  nrow(dat),
-#                  replace = TRUE)
-#
-# library(lavaan)
-# sem_model <-
-# "
-# f1 =~  x1 + x2 + x3
-# f2 =~  x4 + x5 + x6
-# f3 =~  x7 + x8 + x9
-# f2 ~   f1
-# f3 ~   f2
-# "
-#
-# fit_ng <- sem(sem_model, dat)
-# fit_gp <- sem(sem_model, dat, group = "gp")
-#
-# pars <- c("f1 =~ x2", "f2 =~ x5", "f2 ~ f1")
-# tmp <- pars_id(pars, fit = fit_ng)
-# coef(fit_ng)[tmp]
-# tmp <- pars_id(pars, fit = fit_ng, where = "partable")
-# parameterTable(fit_ng)[tmp, ]
-#
-# pars <- c("f1 =~ x2", "f2 =~ c(NA, 1, NA) * x5")
-# tmp <- pars_id(pars, fit = fit_gp)
-# coef(fit_gp)[tmp]
-# tmp <- pars_id(pars, fit = fit_gp, where = "partable")
-# parameterTable(fit_gp)[tmp, ]
-#
-# pars2 <- c("f1 =~ x2", "~~.Beta", "f2 =~ x5.Gamma")
-# tmp <- pars_id(pars2, fit = fit_gp)
-# coef(fit_gp)[tmp]
-# tmp <- pars_id(pars2, fit = fit_gp, where = "partable")
-# parameterTable(fit_gp)[tmp, ]
-# # Note that group 1 is "Beta", not "Alpha"
-# lavInspect(fit_gp, "group.label")
-
-
 #' @title Get id based on lhs-op-rhs, using c() modifiers
 #'
 #' @noRd
@@ -195,12 +200,15 @@ pars_id_lorg_mod <- function(pars,
     if (inherits(parspt, "simpleError")) {
         return(numeric(0))
       }
-    parspt2 <- as.data.frame(lavaan::lavParseModelString(pars))
+    parspt <- parspt[parspt$free > 1, ]
     mcol <- c("lhs", "op", "rhs", "group", "free")
+    mcol2 <- c("lhs", "op", "rhs", "group")
+    parspt <- merge(parspt[, mcol2], ptable[, mcol])
+    parspt2 <- as.data.frame(lavaan::lavParseModelString(pars))
     parspt3 <- merge(parspt[, mcol],
                      parspt2)[, mcol]
     if (free_only) {
-        parspt3 <- parspt3[parspt3$free > 0, ]
+        parspt3 <- parspt3[(parspt3$free > 0) | (parspt3$op == ":="), ]
       }
     parspt4 <- merge(parspt3[, -which(mcol == "free")], ptable)
     if (where == "partable") {
@@ -256,7 +264,7 @@ pars_id_lorg <- function(pars,
     parspt4 <- ptable[ptable$lavlabel %in% pars_c, ]
     if (where == "partable") {
       if (free_only) {
-          out <- parspt4[parspt4$free > 0, "rowid"]
+          out <- parspt4[(parspt4$free > 0) | (parspt4$op == ":="), "rowid"]
         } else {
           out <- parspt4$rowid
         }
@@ -352,7 +360,7 @@ pars_id_op <- function(pars,
     tmp <- ptable[out0, ]
     if (where == "partable") {
         if (free_only) {
-            out <- tmp[tmp$free > 0, "rowid"]
+            out <- tmp[(tmp$free > 0) | (tmp$op == ":="), "rowid"]
           } else {
             out <- tmp$rowid
           }
@@ -373,6 +381,52 @@ keep_ops <- function(pars, ops) {
     unique(pars[unlist(tmp)])
   }
 
+#' @title Get id based on labels
+#' @noRd
+
+pars_id_label <- function(pars,
+                          fit,
+                          where = c("coef",
+                                    "partable"),
+                          free_only = TRUE) {
+    where <- match.arg(where)
+    pfree <- lavaan::lavInspect(fit, "npar")
+    ngp <- lavaan::lavInspect(fit, "ngroups")
+    glabels <- lavaan::lavInspect(fit, "group.label")
+    ptable <- lavaan::parameterTable(fit)
+    # Keep user-supplied labels
+    tmp1 <- ptable$plabel[ptable$plabel != ""]
+    tmp2 <- ptable$label %in% tmp1
+    ptable$label[tmp2] <- ""
+    ptable$rowid <- seq_len(nrow(ptable))
+    ptable$lavlabel <- lavaan::lav_partable_labels(ptable, type = "user")
+    pars_c <- sapply(pars, function(x) {
+                              gsub(x = x,
+                                   pattern = " ",
+                                   replacement = "",
+                                   fixed = TRUE)
+                            }, USE.NAMES = FALSE)
+    out0 <- integer(0)
+    # For both operators without suffixes
+    # and models with only one group
+    tmp <- ptable[ptable$label %in% pars, ]
+    if (nrow(tmp) == 0) {
+        return(out0)
+      }
+    if (where == "partable") {
+        if (free_only) {
+            out <- tmp[(tmp$free > 0) | (tmp$op == ":="), "rowid"]
+          } else {
+            out <- tmp$rowid
+          }
+      }
+    if (where == "coef") {
+        out <- tmp[tmp$free > 0, "free"]
+      }
+    out
+  }
+
+
 #' @title Get id based on wildcard
 #' @noRd
 
@@ -464,6 +518,28 @@ pars_id_special <- function(pars,
 #' tmp <- pars_id(pars, fit = fit_ng, where = "partable")
 #' pars_id_to_lorg(tmp, pars_source = parameterEstimates(fit_ng))
 #'
+#' # Multiple-group models
+#'
+#' dat$gp <- sample(c("Alpha", "Beta", "Gamma"),
+#'                  nrow(dat),
+#'                  replace = TRUE)
+#'
+#' fit_gp <- sem(sem_model, dat, group = "gp")
+#'
+#' pars <- c("f1 =~ x2", "f2 =~ c(NA, 1, NA) * x5")
+#' tmp <- pars_id(pars, fit = fit_gp)
+#' pars_id_to_lorg(tmp, pars_source = coef(fit_gp))
+#' tmp <- pars_id(pars, fit = fit_gp, where = "partable")
+#' pars_id_to_lorg(tmp, pars_source = parameterEstimates(fit_gp))
+#'
+#' parameterTable(fit_gp)[tmp, ]
+#' pars2 <- c("f1 =~ x2", "~~.Beta", "f2 =~ x5.Gamma")
+#' tmp <- pars_id(pars2, fit = fit_gp)
+#' pars_id_to_lorg(tmp, pars_source = coef(fit_gp))
+#' tmp <- pars_id(pars2, fit = fit_gp, where = "partable")
+#' pars_id_to_lorg(tmp, pars_source = parameterEstimates(fit_gp))
+#' # Note that group 1 is "Beta", not "Alpha"
+#' lavInspect(fit_gp, "group.label")
 #'
 #' @export
 
@@ -498,31 +574,3 @@ pars_id_to_lorg <- function(pars_id,
       }
     out
   }
-
-# ###############
-# # To be added when other functions support
-# # multisample models
-# ###############
-#
-# Example
-#
-# dat$gp <- sample(c("Alpha", "Beta", "Gamma"),
-#                  nrow(dat),
-#                  replace = TRUE)
-#
-# fit_gp <- sem(sem_model, dat, group = "gp")
-#
-# pars <- c("f1 =~ x2", "f2 =~ c(NA, 1, NA) * x5")
-# tmp <- pars_id(pars, fit = fit_gp)
-# pars_id_to_lorg(tmp, pars_source = coef(fit_gp))
-# tmp <- pars_id(pars, fit = fit_gp, where = "partable")
-# pars_id_to_lorg(tmp, pars_source = parameterEstimates(fit_gp))
-#
-# parameterTable(fit_gp)[tmp, ]
-# pars2 <- c("f1 =~ x2", "~~.Beta", "f2 =~ x5.Gamma")
-# tmp <- pars_id(pars2, fit = fit_gp)
-# pars_id_to_lorg(tmp, pars_source = coef(fit_gp))
-# tmp <- pars_id(pars2, fit = fit_gp, where = "partable")
-# pars_id_to_lorg(tmp, pars_source = parameterEstimates(fit_gp))
-# # Note that group 1 is "Beta", not "Alpha"
-# lavInspect(fit_gp, "group.label")
\ No newline at end of file
